@@ -33,6 +33,9 @@
 #' @param scale Logical; If \code{TRUE} (the default), predictors are
 #'   scaled so that the maximum Euclidean distance between two points
 #'   is 1. This often improves sampling speed and convergence.
+#'   Scaling also affects the estimated length-scale parameters
+#'   in that they resemble those of scaled predictors (not of the original
+#'   predictors) if \code{scale} is \code{TRUE}.
 #' @param c Numeric value only used in approximate GPs. Defines the 
 #'   multiplicative constant of the predictors' range over which
 #'   predictions should be computed. A good default could be \code{c = 5/4} 
@@ -81,19 +84,19 @@
 #' # fit a simple GP model
 #' fit1 <- brm(y ~ gp(x2), dat, chains = 2)
 #' summary(fit1)
-#' me1 <- conditional_effects(fit1, nsamples = 200, spaghetti = TRUE)
+#' me1 <- conditional_effects(fit1, ndraws = 200, spaghetti = TRUE)
 #' plot(me1, ask = FALSE, points = TRUE)
 #' 
 #' # fit a more complicated GP model
 #' fit2 <- brm(y ~ gp(x0) + x1 + gp(x2) + x3, dat, chains = 2)
 #' summary(fit2)
-#' me2 <- conditional_effects(fit2, nsamples = 200, spaghetti = TRUE)
+#' me2 <- conditional_effects(fit2, ndraws = 200, spaghetti = TRUE)
 #' plot(me2, ask = FALSE, points = TRUE)
 #' 
 #' # fit a multivariate GP model
 #' fit3 <- brm(y ~ gp(x1, x2), dat, chains = 2)
 #' summary(fit3)
-#' me3 <- conditional_effects(fit3, nsamples = 200, spaghetti = TRUE)
+#' me3 <- conditional_effects(fit3, ndraws = 200, spaghetti = TRUE)
 #' plot(me3, ask = FALSE, points = TRUE)
 #' 
 #' # compare model fit
@@ -212,6 +215,8 @@ tidy_gpef <- function(x, data) {
 # exponential-quadratic covariance matrix
 # not vectorized over parameter values
 cov_exp_quad <- function(x, x_new = NULL, sdgp = 1, lscale = 1) {
+  sdgp <- as.numeric(sdgp)
+  lscale <- as.numeric(lscale)
   Dls <- length(lscale)
   if (Dls == 1L) {
     # one dimensional or isotropic GP
@@ -268,7 +273,7 @@ spd_cov_exp_quad <- function(x, sdgp = 1, lscale = 1) {
     constant <- sdgp^2 * sqrt(2 * pi)^D * matrixStats::rowProds(lscale)
     neg_half_lscale2 = -0.5 * lscale^2
     for (m in seq_len(NB)) {
-      x2 <- as_draws_matrix(x[m, ]^2, dim = dim(lscale))
+      x2 <- data2draws(x[m, ]^2, dim = dim(lscale))
       out[, m] <- constant * exp(rowSums(neg_half_lscale2 * x2))
     }
   }
